////////////////////////////////////////////////////////////////////
//!
//! KS-10 Processor
//!
//! \brief
//!      Reset Generator
//!
//! \details
//!      Some devices require that the reset signal is asserted for
//!      a few clock cycles before reset is negated.  Also the
//!      microsequencer design requires that the reset signal
//!      is negated synchronously.
//!
//! \note
//!      Reset is asynchronously asserted and synchronously negated.
//!
//! \todo
//!
//! \file
//!      ks10.v
//!
//! \author
//!      Rob Doyle - doyle (at) cox (dot) net
//!
////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2012 Rob Doyle
//
// This source file may be used and distributed without
// restriction provided that this copyright statement is not
// removed from the file and that any derivative work contains
// the original copyright notice and the associated disclaimer.
//
// This source file is free software; you can redistribute it
// and/or modify it under the terms of the GNU Lesser General
// Public License as published by the Free Software Foundation;
// version 2.1 of the License.
//
// This source is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General
// Public License along with this source; if not, download it
// from http://www.gnu.org/licenses/lgpl.txt
//
////////////////////////////////////////////////////////////////////
//
// Comments are formatted for doxygen
//

module RST(clk, reset, rst);

   input  clk;                  // Clock
   input  reset;                // Reset Input
   output rst;

   //
   // Reset
   //
   // Details
   //  Reset is asynchronously asserted and synchronously negated.
   //

   reg r1;
   reg r2;
   reg r3;

   always @(posedge clk or posedge reset)
     begin
        if (reset)
          begin
             r1 <= 1'b1;
             r2 <= 1'b1;
             r3 <= 1'b1;
          end
        else
          begin
             r1 <= 1'b0;
             r2 <= r1;
             r3 <= r2;
          end
     end

   assign rst = r3;

endmodule


